/*
 *  $Id: device.cc,v 1.1.1.1 1997/10/25 23:11:08 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#include "misc/util.h"

#include "vc1541/device.h"
#include "vc1541/dev_dos.h"
#include "vc1541/fs_fact.h"

Device::Device(FSFactory *fs_factory, int id)
{
	_id = id;
	_fs_factory = fs_factory;
	_root = _fs_factory->get_fs(NULL, NULL, NULL);
	_fs.add(_root);
}

int Device::id(void)
{
	return _id;
}

Directory * Device::readdir(void)
{
	return _fs.top()->get_dir();
}

char * Device::fstype(void)
{
	return _fs.top()->fstype();
}

char * Device::path(void)
{
        char *buf, *p;
	int a, len, maxlen, size;

        maxlen = 1024;
        buf = new char[maxlen];

	len = 1;
        buf[0] = '\0';
	size = _fs.size();
        for (a = 0;a < size;a++) {
        	if ((p = _fs[a]->get_path()) != NULL) {
			len += strlen(p) + 1;
	                if (len >= maxlen) {
		                while (maxlen < len) maxlen *= 2;
	                        char *tmp = new char[maxlen];
	                        strcpy(tmp, buf);
	                        delete buf;
	                        buf = tmp;
	                }
	                strcat(buf, p);
	                strcat(buf, "/");
		}
        }

	return buf;
}

File * Device::open(char *name)
{
	Directory *dir;
	DirectoryEntry *e, *de;

	e = NULL;
        dir = readdir();
	for (de = dir->first();de != NULL;de = dir->next(de)) {
		if (util_glob(name, de->name())) {
			e = de;
			break;
		}
	}
	if (e == NULL) return NULL;

	return _fs.top()->open(path(), e);
}

/*
 *  return value:
 *    -1: error
 *   >=0: active index
 */
int Device::chdir(char *path)
{
	int size;
	Directory *dir;
        FileSystem *new_fs;
	DirectoryEntry *e, *de;

	if (strlen(path) < 1) return -1;
        if (strcmp(path, ".") == 0) return 0;
        if (strcmp(path, "..") == 0) {
        	size = _fs.size();
        	if (size > 1) {
                	/*
                         *  don't remove the root file system
                         */
			new_fs = _fs.remove();
                        delete new_fs;
			de = _fs.top()->chdir(NULL);
			return de->index();
                }
                return 0;
        }

	e = NULL;
        dir = readdir();
	for (de = dir->first();de;de = dir->next(de)) {
		/*
		 *  FIXME: globbing goes here!
		 */
		if (strcmp(path, de->name()) == 0) {
			e = de;
			break;
		}
	}
	if (e == NULL) return -1;

        new_fs = _fs_factory->get_fs(this, _fs.top(), e);
        if (new_fs == NULL) return -1;

        _fs.top()->chdir(e);
        _fs.add(new_fs);

        return 0;
}

