/*
 *  $Id: file.cc,v 1.1.1.1 1997/10/25 23:11:08 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#include <unistd.h>
#include <sys/stat.h>

#include "misc/util.h"
#include "misc/debug.h"
#include "misc/assert.h"
#include "vc1541/file.h"

TmpFile::TmpFile(char *path, char *name, file_mode_t mode) : File(path, name, mode)
{
}

TmpFile::~TmpFile(void)
{
	*debug << "*** File del:   " << _buf << "\n";
	if (_f != NULL) {
        	fclose(_f);
		unlink(_buf);
                _f = NULL;
        }
}

void File::check_gzip(void)
{
	int a;
        char buf[2];
        char cmd[1024];
        char tmp_name[L_tmpnam];
	struct stat statbuf;

        a = fread(buf, 1, 2, _f);
        ::rewind(_f);

        if (a != 2) return;
	if (strncmp(buf, "\x1f\x8b", 2) != 0) return;

       	fclose(_f);

	tmpnam(tmp_name);
#ifdef MSDOS
	sprintf(cmd, "gzip -dc \"%s\" > %s", _buf, tmp_name);
#endif /* MSDOS */
#ifdef LINUX
	sprintf(cmd, "gzip -dc '%s' > '%s'", _buf, tmp_name);
#endif /* LINUX */
	if (util_system(cmd) == 0) {
	        if (stat(tmp_name, &statbuf) != 0) {
			_ok = _reg = false;
                        return;
                }
		_size = statbuf.st_size;
                if ((_f = fopen(tmp_name, "rb")) != NULL) {
                	_realpath = new char [strlen(tmp_name) + 1];
                        strcpy(_realpath, tmp_name);
			*debug << "check_gzip(): GZIP ok (name: " << _realpath
 			       << ")\n";
	                return;
                }
        }

	*debug << "check_gzip(): GZIP failed\n";
        _ok = false;
        _reg = false;
}

File::File(char *path, char *name, file_mode_t mode)
{
#ifdef MSDOS
	int a;
        char *tmp;
#endif
	struct stat statbuf;
	int len, plen, nlen;

	_realpath = NULL;
	ASSERT(path != NULL);
	plen = strlen(path);
	_path = new char[plen + 1];
        strcpy(_path, path);
        if (name == NULL) {
		_name = "<noname>";
                nlen = 0;
        } else {
        	nlen = strlen(name);
        	_name = new char[nlen + 1];
                strcpy(_name, name);
        }
        len = plen + nlen;
        _buf = new char[len + 2];
        strcpy(_buf, _path);
        if (name != NULL) {
        	strcat(_buf, _name);
        }

#ifdef MSDOS
	/*
	 *  convert / to \
	 */
        for (a = 0;a < len;a++) {
        	if (_buf[a] == '/') _buf[a] = '\\';
        }
#endif /* MSDOS */

#ifdef MSDOS
	/*
	 *  check for c: -> c:\
	 */
	tmp = strchr(_buf, ':');
	ASSERT(tmp != NULL);
	if (((tmp - _buf) + 1) == len) {
		strcat(_buf, "\\");
	}
#endif /* MSDOS */

	_f = NULL;
	_size = 0;
        _ok = _dir = _reg = false;

	*debug << "*** File open:  " << _buf << "\n";
        if (stat(_buf, &statbuf) == 0) {
                if (S_ISDIR(statbuf.st_mode)) _ok = _dir = true;
                if (S_ISREG(statbuf.st_mode)) _ok = _reg = true;
		_size = statbuf.st_size;
	}

        switch (mode) {
        case File_RO:   _mode = "rb";  break;
        case File_WR:   _mode = "wb";  break;
        case File_RDWR: _mode = "wb+"; break;
        }

        if ((_f = fopen(_buf, _mode)) == NULL) {
                _ok = _reg = false;
        } else {
                _ok = _reg = true;
                if (mode == File_RO) check_gzip();
        }
}

File::~File(void)
{
	*debug << "*** File close: " << _buf << "\n";
	if (_f != NULL) fclose(_f);
        if (_realpath != NULL) {
		*debug << "*** GZIP del:   " << _realpath << "\n";
                unlink(_realpath);
        	delete _realpath;
        }
	delete _name;
        delete _path;
        delete _buf;
}

void File::rewind(void)
{
	if (_f != NULL) {
	   ::rewind(_f);
   }
}

bool File::seek(long offset)
{
	ASSERT(_f != NULL);
	ASSERT(offset >= 0);
	if (fseek(_f, offset, SEEK_SET) == 0) return true;
	return false;
}

long File::read(void *buf, long len)
{
	ASSERT(_f != NULL);
	ASSERT(buf != NULL);
	ASSERT(len > 0);
	return fread(buf, 1, len, _f);
}

long File::write(void *buf, long len)
{
	ASSERT(_f != NULL);
        ASSERT(buf != NULL);
        ASSERT(len > 0);
        return ::fwrite(buf, 1, len, _f);
}

int File::getc(void)
{
	ASSERT(_f != NULL);
	return ::fgetc(_f);
}

char * File::fgets(void *buf, long maxlen)
{
	ASSERT(_f != NULL);
        ASSERT(buf != NULL);
        ASSERT(maxlen > 0);
        return ::fgets((char *)buf, maxlen, _f);
}

char * File::dir(void)
{
	return _path;
}

char * File::name(void)
{
	return _name;
}

char * File::path(void)
{
	return _buf;
}

char * File::realpath(void)
{
	if (_realpath) return _realpath;
        return _buf;
}

long File::size(void)
{
	return _size;
}

bool File::ok(void)
{
	return _ok;
}

bool File::is_regular(void)
{
	return _reg;
}

bool File::is_directory(void)
{
	return _dir;
}

