/*
 *  $Id: fs_arj.cc,v 1.1.1.1 1997/10/25 23:11:07 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "misc/util.h"
#include "misc/debug.h"
#include "vc1541/fs_arj.h"

void FileSystemARJ::parse(char *filename)
{
	int a;
	File *f;
        int state;
        int blocks;
	char buf[1024];

	f = new TmpFile(filename);
        if (!f->ok()) {
        	delete f;
                return;
        }

        state = 0;
	while (1) {
		if (f->fgets(buf, 1024) == NULL) {
                        delete f;
			return;
                }
                switch (state) {
                case 0:
                	if (buf[1] == '-') state = 1;
                        break;
		case 1:
                	if (buf[1] == '-') {
                                delete f;
                		return;
                        }
                        buf[12] = '\0';
                        buf[23] = '\0';
                        for (a = 11;a > 0;a--) {
                        	if (buf[a] == ' ') {
                        		buf[a] = '\0';
                                        break;
                                }
                        }
                        blocks = atol(buf + 13);
                        if (blocks > 0) blocks = blocks / 254 + 1;
			_dir->add(buf, blocks, " PRG ");
                }
        }
}

FileSystemARJ::FileSystemARJ(File *file) :
 		FileSystemEXEC(file, "External Archiver (ARJ)")
{
	char buf[1024];
        char tmp_name[L_tmpnam];

	tmpnam(tmp_name);

       	sprintf(buf,
#ifdef MSDOS
		"arj l %s > %s",
#endif /* MSDOS */
#ifdef LINUX
		"unarj l '%s' > '%s'",
#endif /* LINUX */
		file->realpath(), tmp_name);
        util_system(buf);
	parse(tmp_name);
}

File * FileSystemARJ::open(char * /*path*/, DirectoryEntry *e)
{
	char buf[1024];
        char tmp_name[L_tmpnam];

#ifdef LINUX
	return NULL; /* fixme: implement arj decompression for linux */
#endif /* LINUX */

	tmpnam(tmp_name);

#ifdef MSDOS
       	sprintf(buf, "arj p -ja1 -i %s %s > %s",
               	_file->realpath(), e->name(), tmp_name);
#endif /* MSDOS */

        util_system(buf);

        return new TmpFile(tmp_name);
}

FileSystem * FileSystemARJ::check(File *file)
{
        if (check_type_exec(file, 0, 2, "\x60\xea")) {
		*debug << "FileSystemARJ::check(): ok... " << file->path() << "\n";
		return new FileSystemARJ(file);
        }
	*debug << "FileSystemARJ::check(): failed!\n";
        return NULL;
}
