/*
 *  $Id: fs_d64.cc,v 1.1.1.1 1997/10/25 23:11:08 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#include "misc/debug.h"
#include "vc1541/fs_d64.h"

static int d64_read_sector(byte_t *buf, int track, int sector, File *file)
{
	long a, offset;
	static int stab[] = {
		21, 21, 21, 21, 21, 21, 21, 21, 21, 21,
		21, 21, 21, 21, 21, 21, 21,
		19, 19, 19, 19, 19, 19, 19,
		18, 18, 18, 18, 18, 18,
		17, 17, 17, 17, 17
	};

	if (track <= 0 || track > 35) return 0;
	if (sector < 0 || sector >= stab[track - 1]) return 0;
	offset = 0;
	for (a = 0;a < track - 1;a++) offset += stab[a];
	offset = (offset + sector) * 256;
	if (!file->seek(offset)) return 0;
	if (file->read((char *)buf, 256) != 256) return 0;

	return 1;
}

static int d64_free_blocks(unsigned char *buf)
{
	int a, b;

	/*
	 *  count free block in BAM but
	 *  ignore the directory track 18
	 */
	b = 0;
	for (a = 4;a < 143;a += 4) {
		if (a != 72) b += buf[a];
	}

	return b;
}

static int d64_find_file(char *filename, File *file, char *filename_ret,
			byte_t *track, byte_t *sector, word_t *blocks)
{
	int a, b;
	char fname[17];
	byte_t buf[256];

	d64_read_sector(buf, 18, 0, file);
	do {
		d64_read_sector(buf, buf[0], buf[1], file);
		for (a = 0;a < 256;a += 32) {
			/*
			 *  ignore everything else than normal prg file
			 *  (but it may be protected
			 */
			if ((buf[a + 2] & 0xbf) == 0x82) {
				for (b = 0;b < 16 && buf[a + b + 5] != 0xa0;b++) {
					fname[b] = buf[a + b + 5];
				}
				fname[b] = '\0';
				/* if (util_glob_file(filename, fname, 0)) {*/
				if (strcmp(filename, fname) == 0) {
					*track = buf[a + 3];
					*sector = buf[a + 4];
					*blocks = buf[a + 30] + 256 * buf[a + 31];
					strcpy(filename_ret, fname);
					return 1;
				}
			}
		}
	} while (buf[0] != 0);
	return 0;
}

FileSystemD64::FileSystemD64(File *file) : FileSystem(file, "Disk Image (D64)")
{
	byte_t buf[256];
	char name[23];
	char type[6];
	word_t a, c, blocks, free;

	if (!d64_read_sector(buf, 18, 0, file)) return;

	for (a = 0;a < 16;a++) {
		name[a] = buf[144 + a];
	}
	name[a++] = ' ';
	name[a++] = buf[162];			               /* disk id */
	name[a++] = buf[163];			    	       /* disk id */
	name[a++] = buf[164] & 0x7f;			 /* shifted space */
	name[a++] = buf[165];		   	   /* dos version/ format */
	name[a++] = buf[166];		   	   /* dos version/ format */
	name[a]   = '\0';
	free = d64_free_blocks(buf);

	_dir->set_title(name, free);

	do {
		if (buf[0] != 18) break;
		if (!d64_read_sector(buf, buf[0], buf[1], file)) break;
		for (a = 0;a < 256;a += 32) {
			if ((buf[a + 2] & 0x8f) == 0) continue;
			blocks = buf[a + 30] + 256 * buf[a + 31];

			for (c = 0;c < 16;c++) {
				if (buf[a + 5 + c] == 0xa0) {
					break;
				}
				name[c] = buf[a + 5 + c];
			}
			name[c] = '\0';

			/*
			 *  deleted file ?
			 */
			if (buf[a + 2] & 0x80) {
				strcpy(type, " ");
			} else {
				strcpy(type, "*");
			}
			switch (buf[a + 2] & 0x0f) {
			case 0:  strcat(type, "DEL"); break;
			case 1:  strcat(type, "SEQ"); break;
			case 2:  strcat(type, "PRG"); break;
			case 3:  strcat(type, "USR"); break;
			case 4:  strcat(type, "REL"); break;
			default: strcat(type, "???"); break;
			}
			/*
			 *  protected file ?
			 */
			if (buf[a + 2] & 0x40) {
				strcat(type, "<");
			} else {
				strcat(type, " ");
			}
			_dir->add(name, blocks, type);
		}
	} while (buf[0] != 0);
}

File * FileSystemD64::open(char * /*path*/, DirectoryEntry *e)
{
	File *f;
	File *_file;
	char tmp_name[L_tmpnam];
	char fname[17];
	byte_t buf[256];
	word_t a, c, blocks;

	_file = file();
	_file->rewind();

	c = 0;
	if (d64_find_file(e->name(), _file, fname, &buf[0], &buf[1], &blocks)) {

		tmpnam(tmp_name);
	        f = new TmpFile(tmp_name, NULL, File_RDWR);
		if (!f->ok()) {
	        	delete f;
	                return NULL;
	        }

		do {
			c++;
			/*
			printf("[%03d/ %03d: T%02d S%02d] '%s'\n", c,
					blocks, buf[0], buf[1], fname);
					*/
			if (!d64_read_sector(buf, buf[0], buf[1], _file)) {
                        	delete f;
				return 0;
                        }
			if (buf[0] == 0) {
				/*
				 *  last block
				 */
				for (a = 2;a <= buf[1];a++) {
                                	f->write(&buf[a], 1);
				}
                                f->write(&buf[a], 1);
                                f->rewind();
				return f;
			} else {
				for (a = 2;a < 256;a++) {
                                        f->write(&buf[a], 1);
				}
			}
		} while (buf[0] != 0);

	        delete f;
	}

	return NULL;
}

bool FileSystemD64::check_type_d64(File *file)
{
	if (!file->is_regular()) return false;
	if (file->size() != 174848) return false;

	return true;
}

FileSystem * FileSystemD64::check(File *file)
{
	if (check_type_d64(file)) {
		*debug << "FileSystemD64::check(): ok... " << file->path() << "\n";
		return new FileSystemD64(file);
	}
	*debug << "FileSystemD64::check(): failed!\n";
	return NULL;
}
