/*
 *  $Id: fs_fact.cc,v 1.1.1.1 1997/10/25 23:11:07 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#include <malloc.h>

#include "misc/assert.h"
#include "vc1541/fs_fact.h"

#include "vc1541/fs_dos.h"
#include "vc1541/fs_unix.h"
#include "vc1541/fs_d64.h"
#include "vc1541/fs_t64.h"
#include "vc1541/fs_lnx.h"
#include "vc1541/fs_arj.h"
#include "vc1541/fs_lha.h"
#include "vc1541/fs_tar.h"
#include "vc1541/fs_zip.h"
#include "vc1541/fs_zipc.h"
#include "vc1541/dev_dos.h"

FSFactory::FSFactory(void)
{
#ifdef MSDOS
	register_fs(FileSystemDOS::check);
#endif /* MSDOS */
#ifdef LINUX
	register_fs(FileSystemUNIX::check);
#endif /* LINUX */
	register_fs(FileSystemT64::check);
	register_fs(FileSystemLNX::check);
        register_fs(FileSystemARJ::check);
        register_fs(FileSystemLHA::check);
        register_fs(FileSystemTAR::check);
	register_fs(FileSystemZIP::check);
	register_fs(FileSystemZIPC::check);
	register_fs(FileSystemD64::check); /* this should go last */
}

void FSFactory::register_fs(check_func f)
{
	_check.add(f);
}

FileSystem * FSFactory::get_fs(Device *dev, FileSystem *fs, DirectoryEntry *e)
{
        File *f;
	int a, size;
        char *path;
        FileSystem *new_fs;

        if ((dev == NULL) && (fs == NULL) && (e == NULL)) {
		return new FileSystemDevDOS();
        }

        ASSERT(dev != NULL);
        ASSERT(fs != NULL);
        ASSERT(e != NULL);

        path = dev->path();

	f = fs->open(path, e);
        if (f == NULL) return NULL;

	new_fs = NULL;
        size = _check.size();
	for (a = 0;a < size;a++) {
		f->rewind();
        	if ((new_fs = _check[a](f)) != NULL) {
			/*
			 *  do not delete f!
			 */
        		return new_fs;
		}
	}

        delete f;
        return NULL;
}
