/*
 *  $Id: fs_lha.cc,v 1.1.1.1 1997/10/25 23:11:08 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "misc/util.h"
#include "misc/debug.h"
#include "vc1541/fs_lha.h"

void FileSystemLHA::parse(char *filename)
{
	int a;
	File *f;
        int state;
        int blocks;
	char buf[1024];

	f = new TmpFile(filename);
        if (!f->ok()) {
        	delete f;
                return;
        }

        state = 0;
	while (1) {
		if (f->fgets(buf, 1024) == NULL) {
                        delete f;
			return;
                }
                switch (state) {
                case 0:
                	if (buf[1] == '-') state = 1;
                        break;
		case 1:
                	if (buf[1] == '-') {
                                delete f;
                		return;
                        }
                        buf[14] = '\0';
                        buf[26] = '\0';
                        for (a = 13;a > 0;a--) {
                        	if (buf[a] == ' ') {
                        		buf[a] = '\0';
                                        break;
                                }
                        }
                        blocks = atol(buf + 15);
                        if (blocks > 0) blocks = blocks / 254 + 1;
			_dir->add(buf + 2, blocks, " PRG ");
                }
        }
}

FileSystemLHA::FileSystemLHA(File *file) :
 		FileSystemEXEC(file, "External Archiver (LHA)")
{
	char buf[1024];
        char tmp_name[L_tmpnam];

	tmpnam(tmp_name);

       	sprintf(buf,
#ifdef MSDOS
               	"lha l %s > %s",
#endif /* MSDOS */
#ifdef LINUX
               	"lha l '%s' > '%s'",
#endif /* LINUX */
       		file->realpath(), tmp_name);

        util_system(buf, false);

        parse(tmp_name);
}

File * FileSystemLHA::open(char * /*path*/, DirectoryEntry *e)
{
	char buf[1024];
        char tmp_name[L_tmpnam];

#ifdef LINUX
	return NULL; /* fixme: implement lha decompression for linux */
#endif /* LINUX */

	tmpnam(tmp_name);

#ifdef MSDOS
       	sprintf(buf, "lha p /m /n %s %s > %s",
               	_file->realpath(), e->name(), tmp_name);
#endif /* MSDOS */

        util_system(buf, false);

        return new TmpFile(tmp_name);
}

FileSystem * FileSystemLHA::check(File *file)
{
        if (check_type_exec(file, 2, 3, "-lh")) {
		*debug << "FileSystemLHA::check(): ok... " << file->path() << "\n";
		return new FileSystemLHA(file);
        }
	*debug << "FileSystemLHA::check(): failed!\n";
        return NULL;
}

