/*
 *  $Id: gui.cc,v 1.1.1.1 1997/10/25 23:11:08 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#include "gui/gui.h"
#include "gui/guidebug.h"

#include "misc/assert.h"

const short cmMyNewWindow = 199;
const short cmChdir = 200;

#define cpChangeableText "\x06"

TChangeableText::TChangeableText( const TRect& bounds, const char *aText ) :
    TView( bounds ),
    text( newStr( aText ) )
{
}

TChangeableText::~TChangeableText()
{
    delete (char *)text;
}

void TChangeableText::draw()
{
    uchar color;
    Boolean center;
    int i, j, l, p, y;
    TDrawBuffer b;
    char s[256];

    color = getColor(1);
    getText(s);
    l = strlen(s);
    p = 0;
    y = 0;
    center = False;
    while (y < size.y)
        {
        b.moveChar(0, ' ', color, size.x);
        if (p < l)
            {
            if (s[p] == 3)
                {
                center = True;
                ++p;
                }
            i = p;
            do {
               j = p;
               while ((p < l) && (s[p] == ' ')) 
                   ++p;
               while ((p < l) && (s[p] != ' ') && (s[p] != '\n'))
                   ++p;
               } while ((p < l) && (p < i + size.x) && (s[p] != '\n'));
            if (p > i + size.x)
                if (j > i)
                    p = j; 
                else 
                    p = i + size.x;
            if (center == True)
               j = (size.x - p + i) / 2 ;
            else 
               j = 0;
            b.moveBuf(j, &s[i], color, (p - i));
            while ((p < l) && (s[p] == ' '))
                p++;
            if ((p < l) && (s[p] == '\n'))
                {
                center = False;
                p++;
                if ((p < l) && (s[p] == 10))
                    p++;
                }
            }
        writeLine(0, y++, size.x, 1, b);
        }
}

TPalette& TChangeableText::getPalette() const
{
	static TPalette palette(cpChangeableText, sizeof(cpChangeableText) - 1);
	return palette;
}

void TChangeableText::getText(char *s)
{
	if (text == 0) {
		*s = '\0';
	} else {
	        strcpy(s, text);
	}
}

void TChangeableText::setText(char *s)
{
	delete (char *)text;
	text = newStr(s);
	drawView();
}

TDirList::TDirList(const TRect &r, TScrollBar *vScrollBar, Device *dev) :
		TListViewer(r, 1, 0, vScrollBar)
{
	_dev = dev;
	update();
}

void TDirList::update(int idx)
{
	_dir = _dev->readdir();
	setRange(_dir->size());
	focusItem(idx);
	drawView();
}

void TDirList::getText(char *dest, ccIndex item, short maxLen)
{
	int a;
	char buf[100];
        DirectoryEntry *de;

	ASSERT(maxLen > 10);
	de = _dir->first();
	ASSERT(de != NULL);
	for (a = 0;a < item;a++) {
		de = _dir->next(de);
		ASSERT(de != NULL);
	}
	sprintf(buf, "%-5d %-16.16s %-5.5s", de->blocks(), de->name(), de->type());
	maxLen--;
	strncpy(dest, buf, maxLen);
	dest[maxLen] = '\0';
}

char * TDirList::getSelected(void)
{
	int a;
        DirectoryEntry *de;
	de = _dir->first();
	ASSERT(de != NULL);
	for (a = 0;a < focused;a++) {
		de = _dir->next(de);
		ASSERT(de != NULL);
	}
	return de->name();
}

void TDirList::handleEvent(TEvent &event)
{
	TListViewer::handleEvent(event);
	if (event.what == evKeyDown) {
		switch (event.keyDown.keyCode) {
		case kbRight:
			selectItem(focused);
		        clearEvent(event);
			break;
		case kbLeft:
			event.what = evCommand;
	                event.message.command = cmChdir;
        	        event.message.infoPtr =	"..";
	                putEvent(event);
		        clearEvent(event);
			break;
		}
	}
        if (event.what == evBroadcast) {
        	switch (event.message.command) {
                case cmListItemSelected:
	                event.what = evCommand;
	                event.message.command = cmChdir;
        	        event.message.infoPtr =	getSelected();
	                putEvent(event);
		        clearEvent(event);
                        break;
		}
	}
}

App::App(Device *dev) :
	GUI(dev),
	TProgInit(&App::initStatusLine, &App::initMenuBar, &App::initDeskTop)
{
	eventMask |= evBroadcast;
	options |= ofPostProcess;

	_listbox = NULL;

	myNewWindow();
}

void App::myNewWindow(void)
{
	TRect r;

	if (_listbox != NULL) return;

        r = deskTop->getExtent();
        r.a.y = r.b.y - 7;
        TDebugWindow *dw = new TDebugWindow(r);
        dw->flags &= wfClose;
        debug = new debugstream(dw);
        deskTop->insert(dw);

	r = deskTop->getExtent();
	r.b.y = r.b.y - 7;
        r.b.x /= 2;
	TDialog *v1 = new TDialog(r, "Device 8");
	v1->flags &= ~wfClose;

	r = v1->getExtent();
	r.grow(-2, -2);
	r.b.x--;
	TRect r2(r);
	r2.a.x = r2.b.x++;
        TScrollBar *v4 = new TScrollBar(r2);
        _listbox = new TDirList(r, v4, _dev);
	_label = new TChangeableText(TRect(2, 1, 30, 2), "");
        r.a.y = r.b.y++;
        _free = new TChangeableText(r, "");

        v1->insert(v4);
        v1->insert(_label);
        v1->insert(_listbox);
        v1->insert(_free);

	TEvent event;
#ifdef MSDOS
	event.what = evCommand; event.message.command = cmChdir;
	event.message.infoPtr =	"C:";		handleEvent(event);
	event.what = evCommand; event.message.command = cmChdir;
	event.message.infoPtr =	"EMU";		handleEvent(event);
	event.what = evCommand; event.message.command = cmChdir;
	event.message.infoPtr =	"C64";		handleEvent(event);
	event.what = evCommand; event.message.command = cmChdir;
	event.message.infoPtr =	"DATA";		handleEvent(event);
#endif /* MSDOS */
#ifdef LINUX
	event.what = evCommand; event.message.command = cmChdir;
	event.message.infoPtr =	"/";         handleEvent(event);
	event.what = evCommand; event.message.command = cmChdir;
	event.message.infoPtr =	"home";      handleEvent(event);
	event.what = evCommand; event.message.command = cmChdir;
	event.message.infoPtr =	"paul";      handleEvent(event);
	event.what = evCommand; event.message.command = cmChdir;
	event.message.infoPtr =	"ftp";       handleEvent(event);
	event.what = evCommand; event.message.command = cmChdir;
	event.message.infoPtr =	"c64";       handleEvent(event);
#endif /* LINUX */

        deskTop->insert(v1);
}

void App::handleEvent(TEvent &event)
{
	int idx;
        char buf[100];

	TApplication::handleEvent(event);
        if (event.what == evCommand) {
        	switch (event.message.command) {
                case cmMyNewWindow:
                	myNewWindow();
		        clearEvent(event);
                        break;
		case cmChdir:
			if ((idx = _dev->chdir((char *)event.message.infoPtr)) >= 0) {
				_listbox->update(idx);
				_label->setText(_dev->fstype());
                                sprintf(buf, "%ld blocks free.", _dev->readdir()->free());
                                _free->setText(buf);
			}
		        clearEvent(event);
			break;
		}
	}
}

TMenuBar * App::initMenuBar(TRect r)
{
 r.b.y = r.a.y + 1;
 return new TMenuBar(r,
	* new TSubMenu("~F~ile", kbAltF) +
	        * new TMenuItem("E~x~it", cmQuit, kbNoKey, hcNoContext, 0));
}

TStatusLine * App::initStatusLine(TRect r)
{
	r.a.y = r.b.y - 1;
        return new TStatusLine(r,
        	*new TStatusDef(0, 0xffff) +
                *new TStatusItem("~Alt-X~ Exit", kbAltX, cmQuit) +
                *new TStatusItem(0, kbF10, cmMenu) +
                *new TStatusItem(0, kbAltF3, cmClose) +
                *new TStatusItem(0, kbF5, cmZoom) +
                *new TStatusItem(0, kbCtrlF5, cmResize) +
                *new TStatusItem(0, kbF6, cmNext) +
                *new TStatusItem(0, kbShiftF6, cmPrev));
}

bool App::poll(void)
{
	TEvent e;
	getEvent(e);
	handleEvent(e);
	if (e.what != evNothing) eventError(e);
	if (endState == 0) return true;
	if (!valid(endState)) return true;
	return false;
}

