/*
 *  $Id: dir.h,v 1.1.1.1 1997/10/25 23:11:09 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#ifndef __vc1541_dir_h
#define __vc1541_dir_h

#include <stdio.h>
#include <string.h>

#include "vc1541/common.h"

class DirectoryEntry
{
protected:
	int             _idx;
	char           *_name;
	char           *_type;
	word_t          _blocks;
	DirectoryEntry *_n;

public:
	DirectoryEntry(char *name, word_t blocks, char *type, int idx)
	{
		_n = NULL;
		_idx = idx;
		_name = new char[strlen(name) + 1];
		_type = new char[strlen(type) + 1];
		strcpy(_name, name);
		strcpy(_type, type);
		_blocks = blocks;
	}

	~DirectoryEntry(void)
	{
		delete _name;
		delete _type;
		delete _n;
	}

	DirectoryEntry * add(char *name, word_t blocks, char *type, int idx)
	{
		_n = new DirectoryEntry(name, blocks, type, idx);

		return _n;
	}

	DirectoryEntry * next(void)
	{
		return _n;
	}

	char * name(void) { return _name; }
	char * type(void) { return _type; }
	word_t blocks(void) { return _blocks; }
	int    index(void) { return _idx; }
};

class Directory
{
private:
	int _size;
	char *_title;
	dword_t _free;

protected:
	DirectoryEntry *_dir, *_last;

public:
	Directory(void)
	{
		_free = 0;
		_size = 0;
		_title = new char[1];
		_title[0] = '\0';
		_dir = _last = NULL;
	}

	~Directory(void)
	{
		delete _dir;
	}

	void set_title(char *title, dword_t free)
	{
		if (_title != NULL) delete _title;
		_title = new char[strlen(title) + 1];
		strcpy(_title, title);
		_free = free;
	}

	char * title(void) { return _title; }
	dword_t free(void) { return _free; }
	int size(void)     { return _size; }

	void add(char *name, word_t blocks, char *type)
	{
		if (_last == NULL) {
			_dir = _last = new DirectoryEntry(name, blocks, type, _size);
		} else {
			_last = _last->add(name, blocks, type, _size);
		}
		_size++;
	}

	DirectoryEntry * first(void)
	{
		return _dir;
	}

	DirectoryEntry * next(DirectoryEntry *e)
	{
		return e->next();
	}
};

#endif /* __vc1541_dir_h */

