/*
 *  $Id: protocol.cc,v 1.1.1.1 1997/10/25 23:11:08 tp Exp $
 *
 *
 *  Copyright (C) 1997 Torsten Paul
 *
 *  This file is part of VC1541, the Commodore Floppy 1541 Emulator.
 */

#include <ctype.h>
#include <iostream.h>
#include <iomanip.h>

#ifdef MSDOS
#include <pc.h>
#endif /* MSDOS */
#ifdef LINUX
#include <asm/io.h>
#define inportb(port) inb(port)
#define outportb(port,val) outb(val,port)
#endif

#include "vc1541/io.h"
#include "vc1541/protocol.h"

#include "misc/debug.h"

#include "vc1541/dir.h"

#ifdef MSDOS
#define USE_CLI
#endif

#ifdef MSDOS
static unsigned int addr = 0xb8000;

void DPRINT(void *s)
{
#if 1
	unsigned short a = 0x0f00 | (*((char *)s) & 0xff);

	asm (
		"movw %%ax, %%gs:(%%ebx)\n\t"
		: /* no output */
		: "a" ((unsigned short)a), "b" (addr)
	);
	addr = (addr + 2) & 0xb807f;
#endif
}

void IDLE(void)
{
#if 1
	static char s[] = "|/-\\";
	static int count = 0;
	unsigned short a = 0x0f00 | s[count];

        asm (
        	"movw %%ax, %%gs:(%%ebx)\n\t"
                : /* no output */
                : "a" ((unsigned short)a), "b" (addr)
        );
        if (++count > 3) count = 0;
#endif
}
#endif /* MSDOS */

#ifdef LINUX
void DPRINT(void * /* s */)
{
}

void IDLE(void)
{
}
#endif /* LINUX */

Protocol::Protocol(IO *io, GUI *gui, Device *dev)
{
	_io  = io;
	_gui = gui;
	_dev = dev;

        _deferred_delete = NULL;
}

int Protocol::getIEC(byte_t *ret)
{
	int eoi;
	byte_t b;
	byte_t DATA     = _io->DATA;
	byte_t CLOCK    = _io->CLOCK;
	byte_t DATA_BIT = _io->DATA_BIT;

#ifdef USE_CLI
	asm ("cli\n\t");
#endif /* USE_CLI */

	eoi = 0;
	if (_io->wait(CLOCK, 0)) return 0;                    	       /* [$e9d5] */

	_io->set(DATA, 0);                                    	       /* [$e9d7] */
	if (_io->wait(DATA, 0)) return 0;   /* [$ff25] (patch called from $e9dc) */

	/*
	 *  check for eoi
	 */
	if (_io->waitEOI(_io->CLOCK, _io->CLOCK)) {                        /* [$e9ee] */
		eoi = 1;
		DPRINT("*");
		_io->set(_io->DATA, _io->DATA);                        	       /* [$e9f2] */
		_io->microSleep(80);                               	       /* [$e9f8] */
		_io->set(_io->DATA, 0);                                        /* [$e9fa] */
		if (_io->wait(_io->CLOCK, _io->CLOCK)) return 0;               /* [$ea05] */
	}

	if (_io->wait(CLOCK, 0)) return 0;                    /* [$ea13] */
	b = (_io->get(DATA) >> DATA_BIT);
	if (_io->wait(CLOCK, CLOCK)) return 0;           /* [$ea22] */

	if (_io->wait(CLOCK, 0)) return 0;                    /* [$ea13] */
	b |= (_io->get(DATA) >> DATA_BIT) << 1;
	if (_io->wait(CLOCK, CLOCK)) return 0;           /* [$ea22] */

	if (_io->wait(CLOCK, 0)) return 0;                    /* [$ea13] */
	b |= (_io->get(DATA) >> DATA_BIT) << 2;
	if (_io->wait(CLOCK, CLOCK)) return 0;           /* [$ea22] */

	if (_io->wait(CLOCK, 0)) return 0;                    /* [$ea13] */
	b |= (_io->get(DATA) >> DATA_BIT) << 3;
	if (_io->wait(CLOCK, CLOCK)) return 0;           /* [$ea22] */

	if (_io->wait(CLOCK, 0)) return 0;                    /* [$ea13] */
	b |= (_io->get(DATA) >> DATA_BIT) << 4;
	if (_io->wait(CLOCK, CLOCK)) return 0;           /* [$ea22] */

	if (_io->wait(CLOCK, 0)) return 0;                    /* [$ea13] */
	b |= (_io->get(DATA) >> DATA_BIT) << 5;
	if (_io->wait(CLOCK, CLOCK)) return 0;           /* [$ea22] */

	if (_io->wait(CLOCK, 0)) return 0;                    /* [$ea13] */
	b |= (_io->get(DATA) >> DATA_BIT) << 6;
	if (_io->wait(CLOCK, CLOCK)) return 0;           /* [$ea22] */

	if (_io->wait(CLOCK, 0)) return 0;                    /* [$ea13] */
	b |= (_io->get(DATA) >> DATA_BIT) << 7;
	if (_io->wait(CLOCK, CLOCK)) return 0;           /* [$ea22] */

	_io->set(DATA, DATA);                                    /* [$ea28] */

	*ret = ~b;  	                   /* all bits we got need to be inverted */

	DPRINT(ret);

	if (eoi) {
		_io->set(CLOCK, CLOCK);                              /* c64: [$ee0d] */
		_io->set(DATA, DATA);                                /* c64: [$ee10] */
	}
	_io->microSleep(100);         	                      	  /* c64: [$ee0a] */

#ifdef USE_CLI
	asm("sti\n\t");
#endif

	return eoi;
}

int Protocol::putIEC(byte_t b, int eoi)
{
    int a;
    byte_t tmp;
    int flag = 0;

    /* DELAY c64: [$ee1b] */
#ifdef USE_CLI
	asm ("cli\n\t");
#endif /* USE_CLI */

    DPRINT(&b);
    _io->microSleep(100);

    tmp = _io->get(_io->DATA);                                           /* [$e919] */
    _io->set(_io->CLOCK, 0);                    			       /* [$e91f] */

    if (tmp) {                     			       /* [$e923] */
	if (_io->wait(_io->DATA, 0)) return 0;			       /* [$e92d] */
	flag = eoi;
	if (eoi) DPRINT("e");
    } else {
	DPRINT("=");
	DPRINT("1");
	flag = 1;              		    /* [$e923] force eoi, why ??? */
    }
    if (flag) {
	DPRINT("=");
	DPRINT("2");
	if (_io->wait(_io->DATA, 0)) return 0;			       /* [$e93f] */
	if (_io->wait(_io->DATA, _io->DATA)) return 0;			       /* [$e949] */
    }

    _io->set(_io->CLOCK, _io->CLOCK);        				       /* [$e94b] */
    if (_io->wait(_io->DATA, 0)) return 0; 			       /* [$e956] */

    for (a = 0;a < 8;a++) {

	/* DELAY c64: [$ee5a] */
	_io->microSleep(70);

	if (_io->get(_io->DATA)) {
	    DPRINT("~");
	    return 0;					       /* [$e961] */
	}
	if (b & (1 << a)) {
	    _io->set(_io->DATA, 0);     				       /* [$e973] */
	} else {
	    _io->set(_io->DATA, _io->DATA);   				       /* [$e96e] */
	}
	_io->set(_io->CLOCK, 0);         				       /* [$e976] */
	_io->microSleep(70);       	       /* [$e97d -> $fef3] delay for 1541 */

	/* DELAY c64: [$ee67] */

	_io->set(_io->CLOCK, _io->CLOCK);     			      /* [$e980 -> $fefb] */
	_io->set(_io->DATA, 0);   		              /* [$e980 -> $fefb] */
    }
    _io->microSleep(70);
    if (_io->wait(_io->DATA, _io->DATA)) return 0;         		       /* [$e98f] */

#ifdef USE_CLI
	asm("sti\n\t");
#endif

    return flag;
}

void Protocol::sendDirLine(word_t *addr, DirectoryEntry *de)
{
	word_t blocks;
	byte_t a, b, len;
	char name[17], *type;

	type   = de->type();
	blocks = de->blocks();
	sprintf(name, "\"%s\"", de->name());

	b = 1;
	if (blocks < 100) b++;
	if (blocks <  10) b++;
	len = strlen(name);
	*addr += 26 + b;

	putIEC(*addr);
	putIEC(*addr >> 8);
	putIEC(blocks);
	putIEC(blocks >> 8);
	if (_io->_timeout) return;
	for (a = 0;a < b;a++)        putIEC(' ');
	if (_io->_timeout) return;
	for (a = 0;a < len;a++)	     putIEC(name[a]);
	if (_io->_timeout) return;
	for (a = 0;a < 18 - len;a++) putIEC(' ', 0);
	if (_io->_timeout) return;
	for (a = 0;a < 5;a++)	     putIEC(type[a]);
	putIEC(0x00);
}

void Protocol::sendDir(Directory *dir)
{
	byte_t c;
	word_t a, len, addr;
	DirectoryEntry *de;

        *debug << "sending directory...\n";

	addr = 0x0801;
	putIEC(addr);				// load address low
	putIEC(addr >> 8);			// load address hi
	if (_io->_timeout) return;

	len = strlen(dir->title());
	addr += len + 5;
	putIEC(addr);				// link pointer low
	putIEC(addr >> 8);			// link pointer high
	putIEC(0x00);				// line number low
	putIEC(0x00);				// line number high
	if (_io->_timeout) return;

	putIEC('\x12');				// reverse
	putIEC('"');				// "
	for (a = 0;a < len;a++) {
		c = toupper(dir->title()[a]);
		if (c == '\\') c = '/';
		putIEC(c);			// directory name
		if (_io->_timeout) return;
	}
	putIEC('"');				// "
	putIEC(0x00);				// end of line
	if (_io->_timeout) return;

	for (de = dir->first();de;de = dir->next(de)) {
		sendDirLine(&addr, de);
		if (_io->_timeout) return;
	}

	putIEC(addr);				// link pointer low
	putIEC(addr >> 8);			// link pointer high
        /*
         *  we can't have more than 65535 blocks free
         */
        if (dir->free() > 0xffff) {
        	putIEC(0xff);			// blocks free low
                putIEC(0xff);			// blocks free high
        } else {
	        putIEC(dir->free() & 0xff);
	        putIEC((dir->free() >> 8) & 0xff);
        }
        for (a = 0;a < 12;a++) {
        	putIEC("BLOCKS FREE."[a]);
        }
	putIEC(0x00);				// end of line

	putIEC(0x00);
	putIEC(0x00, 1);			// basic end, send with eoi
}

void Protocol::sendFile(File *file)
{
	int c2;
	byte_t c1;
        word_t b, c;

	if (file == NULL) return;

        *debug << "sending file (" << file->path() << ")...\n";

	c2 = file->getc();
	if (c2 == EOF) return;

        b = c = 0;
	while (1) {
		c1 = (byte_t)c2;
		c2 = file->getc();
		if (c2 == EOF) {
			if ((c % 8) != 0) debug->form("\n");
			DPRINT("-");
			DPRINT("E");
			DPRINT("O");
			DPRINT("F");
			DPRINT("-");
			putIEC(c1, 1);
			break;
		} else {
			putIEC(c1);
			if (_io->_timeout) return;
		}
		if ((b++ % 254) == 0) {
                	c++;
			debug->form("[%3d]%c", c, ((c % 8) == 0) ? '\n' : ' ');
		}
	}
}

/*
 *  [$ea2e]
 *
 *  If iobuf != NULL we have an open command and the filename
 *  is following. The other case is a data transfer.
 *  FIXME: buffer overflow not tested!
 *
 */
void Protocol::handleLISTEN(Device * /* dev */, int sec_addr, char *iobuf)
{
	int a;
	int eoi;
	byte_t b;
	char *buf, command[256];

	switch (sec_addr) {
	case 0:
		buf = iobuf;
		break;
	case 15:
		buf = command;
		break;
	default:
		buf = NULL;
		break;
	}

	a = 0;
	do {
		if (_io->get(_io->ATN)) {
			DPRINT("*");
			DPRINT("A");
			DPRINT("*");
			return;
		}
		eoi = getIEC(&b);
		if (buf) {
			buf[a++] = b;
			buf[a] = '\0';
		}
		if (_io->_timeout) return;
	} while (!eoi);

	switch (sec_addr) {
	case 0:
        	debug->form("Protocol::handleLISTEN(): open -> '%s'\n",
 			buf?buf:"<null>");
		break;
	case 15:
        	debug->form("Protocol::handleLISTEN(): command channel -> '%s'\n",
 			buf?buf:"<null>");
		break;
	default:
                debug->form("Protocol::handleLISTEN(): secondary address %d -> '%s'\n",
                	sec_addr, buf?buf:"<null>");
		break;
	}
}

void Protocol::handleTALK(Device *dev, int /* sec_addr */, char *s)
{
	File *f;

	if (strcmp("$", s) == 0) {
		sendDir(dev->readdir());
	} else {
		f = dev->open(s);
		if (f) {
			sendFile(f);
			_deferred_delete = f;
		} else {
			debug->form("Protocol::handleTALK(): file not found!\n");
		}
	}
}

void Protocol::handleCLOSE(void)
{
	debug->form("Protocol::handleCLOSE()\n");
}

/*
 *  [$e85b]
 */
void Protocol::handleATN(void)
{
	byte_t b, c;
	static char iobuf[4096]; /* fixme: */
	byte_t mode, sec_addr;

#ifdef USE_CLI
	asm ("cli\n\t");
#endif /* USE_CLI */

	_io->_timeout = 0;

	_io->wait(_io->CLOCK, 0);             			       /* [$e882] */

	DPRINT("a");

	_io->microSleep(200);
	getIEC(&b);	  					     /* get device number */
	if (_io->_timeout) return;

	mode = b & IO::IEC_COMMAND;
	switch (mode) {
	case IO::IEC_UNLISTEN:
		DPRINT("L");
		_io->wait(_io->ATN, 0);
		DPRINT("A");
		return;
	case IO::IEC_UNTALK:
		DPRINT("T");
		_io->wait(_io->ATN, 0);
		DPRINT("A");
		return;
	case IO::IEC_LISTEN:
		DPRINT("l");
		/*
		 *  release bus if an other device is addressed
		 */
		if (_dev->id() != (b & IO::IEC_DEVICE)) {
			DPRINT("-");
			_io->set(_io->DATA | _io->CLOCK, 0);  	       /* [$e8ff] */
			while (_io->wait(_io->ATN, 0));	 	       /* [$e907] */
			DPRINT("A");
			return;
		}
		break;
	case IO::IEC_TALK:
		DPRINT("t");
		/*
		 *  release bus if an other device is addressed
		 */
		if (_dev->id() != (b & IO::IEC_DEVICE)) {
			DPRINT("-");
			_io->set(_io->DATA | _io->CLOCK, 0);  	       /* [$e8ff] */
			while (_io->wait(_io->ATN, 0));	 	       /* [$e907] */
			DPRINT("A");
			return;
		}
		break;
	}

	_io->microSleep(200);
	getIEC(&c);              		 /* get secondary address */

	if (_io->_timeout) return;

#ifdef USE_CLI
	asm("sti\n\t");
#endif

	switch (c & IO::IEC_MODE) {
	case IO::IEC_OPEN:
		_io->wait(_io->ATN, 0);	         			       /* [$e907] */
		DPRINT("A");
		DPRINT("o");
		sec_addr = c & IO::IEC_ADDRESS;
		handleLISTEN(_dev, sec_addr, iobuf);  	      /* [$e8e7 -> $ea2e] */
		DPRINT("O");
		break;
	case IO::IEC_CLOSE:
		_io->wait(_io->ATN, 0);    				       /* [$e907] */
		DPRINT("A");
		DPRINT("c");
		handleCLOSE();         	              /* [$e8ce -> $dac0] */
		DPRINT("C");
		break;
	case IO::IEC_DATA:
		DPRINT("d");
		sec_addr = c & IO::IEC_ADDRESS;
		switch (mode) {
		case IO::IEC_LISTEN:
			_io->wait(_io->ATN, 0);	   		   	       /* [$e907] */
			DPRINT("A");
			handleLISTEN(_dev, sec_addr, NULL);	      /* [$e8e7 -> $ea2e] */
			break;
		case IO::IEC_TALK:
			_io->set(_io->DATA | _io->CLOCK, 0); 			       /* [$e8ff] */
			_io->wait(_io->ATN, 0);	   			       /* [$e907] */
			DPRINT("A");
			_io->microSleep(200); /* FIXME */
			_io->set(_io->DATA, 0);                		       /* [$e8f1] */
			_io->set(_io->CLOCK, _io->CLOCK);           		       /* [$e8f4] */
			_io->microSleep(200); /* FIXME */
			handleTALK(_dev, sec_addr, iobuf);           /* [$e8f7 -> $e909] */
                        _io->microSleep(100);
			_io->set(_io->DATA | _io->CLOCK, 0);
			break;
		}
		DPRINT("D");
	}

	if (!_io->_timeout) return;
	DPRINT("*");
	DPRINT("T");
	DPRINT("*");
}

void Protocol::execute(void)
{
	int first = 1;

#ifdef USE_CLI
	asm ("cli\n\t");
#endif /* USE_CLI */
	while (1) {
		if (first) {
			_io->set(_io->DATA | _io->CLOCK, 0);
			first = 0;
			DPRINT("-");
			DPRINT(">");
			DPRINT(" ");
		}
		if (_io->get(_io->ATN)) {
			first = 1;
			_io->set(_io->CLOCK, 0);		/* [$e86d] */
			_io->set(_io->DATA, _io->DATA);		/* [$e870] */
			handleATN();
#ifdef MSDOS
			if (inportb(0x60) == 1) break;		/* check ESC key */
#endif
			continue;
		}

                if (_deferred_delete) delete _deferred_delete;
                _deferred_delete = NULL;

#ifdef USE_CLI
		asm("sti\n\t");
#endif
		_io->microSleep(100);
		IDLE();
		if (!_gui->poll()) break;
#ifdef MSDOS
		//if (inportb(0x60) == 1) break;		/* check ESC key */
#endif /* MSDOS */
	}
#ifdef USE_CLI
	asm("sti\n\t");
#endif
	_io->set(_io->DATA | _io->CLOCK, 0);
}

